@@ -4,7 +4,7 @@
 #                                                                   #
 #-------------------------------------------------------------------#
 
-process.capability <- function(object, spec.limits, target, std.dev, nsigmas, confidence.level = 0.95, breaks="scott", add.stats=TRUE, print=TRUE, restore.par=TRUE)
+process.capability <- function(object, spec.limits, target, std.dev, nsigmas, confidence.level = 0.95, breaks="scott", add.stats=TRUE, print=TRUE, digits = getOption("digits"), restore.par=TRUE)
 {
 # Computes process capability indices for a qcc object of type "xbar" 
 # and plot the histogram
@@ -22,21 +22,38 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
      std.dev <- object$std.dev
   n <- length(x)
   title <- paste("Process Capability Analysis\nfor", object$data.name)
+
   if (missing(spec.limits))
      stop("specification limits must be provided")
-  if (!length(spec.limits)==2)
-     stop("wrong specification limits format")
-  LSL <- min(spec.limits, na.rm=TRUE)
-  USL <- max(spec.limits, na.rm=TRUE)
+  spec.limits <- as.vector(spec.limits)[1:2]
+  LSL <- spec.limits[1]
+  if(!(is.numeric(LSL) & is.finite(LSL))) LSL <- NA
+  USL <- spec.limits[2]
+  if(!(is.numeric(USL) & is.finite(USL))) USL <- NA
+  if(is.na(LSL) & is.na(USL))
+     stop("invalid specification limits")
 
-  if (missing(target))
-     { target <- mean(spec.limits, na.rm=TRUE) }
-  if (target < LSL | target > USL)
-     warning("target value is not within specification limits...") 
+  has.target <- (!missing(target))
+  if(!has.target) 
+    { target <- mean(spec.limits, na.rm=TRUE)
+      if(!is.na(LSL) & !is.na(USL)) has.target <- TRUE
+  }
+     
+  if (is.na(LSL))
+     { if (target > USL)
+           warning("target value larger than one-sided specification limit...") }
+  if (is.na(USL))
+     { if (target < LSL)
+           warning("target value smaller than one-sided specification limit...") }
+  if (!is.na(LSL) & !is.na(USL))
+     { if (target < LSL || target > USL)
+       warning("target value is not within specification limits...") }
+       
   if (missing(nsigmas))
      if (is.null(object$nsigmas))
         stop("nsigmas not available in the 'qcc' object. Please provide nsigmas.") 
      else  nsigmas <- object$nsigmas
+  
   if (confidence.level < 0 | confidence.level > 1)
      stop("the argument confidence.level must be a value between 0 and 1") 
 
@@ -64,15 +81,17 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
     c(paste(round(100*alpha/2, 1), "%", sep=""),
       paste(round(100*(1-alpha/2), 1), "%", sep=""))
 
-  exp.LSL <- pnorm((LSL-center)/std.dev) * 100
-  if (exp.LSL < 0.01) exp.LSL <- 0
-  exp.USL <- (1-pnorm((USL-center)/std.dev)) * 100
-  if (exp.USL < 0.01) exp.USL <- 0
+  if(is.na(LSL))  exp.LSL <- NA
+  else { exp.LSL <- pnorm((LSL-center)/std.dev) * 100
+         if(exp.LSL < 0.01) exp.LSL <- 0 }
+  if(is.na(USL))  exp.USL <- NA
+  else { exp.USL <- (1-pnorm((USL-center)/std.dev)) * 100
+         if(exp.USL < 0.01) exp.USL <- 0 }
   obs.LSL <- sum(x<LSL)/n * 100
   obs.USL <- sum(x>USL)/n * 100
-  xlim <- range(x, USL, LSL, target)
+  xlim <- range(x, USL, LSL, target, na.rm = TRUE)
   xlim <- xlim+diff(xlim)*c(-0.1,0.1)
-  xx <- seq(min(xlim), max(xlim), length=100)
+  xx <- seq(min(xlim), max(xlim), length=250)
   dx <- dnorm(xx, center, std.dev)
   h <- hist(x, breaks = breaks, plot=FALSE) # compute histogram
   ylim <- range(h$density, dx)
@@ -84,14 +103,14 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
   rownames(tab) <- c("Cp", "Cp_l", "Cp_u", "Cp_k", "Cpm")
   colnames(tab) <- c("Value", names(Cp.limits))
 
-  oldpar <- par(bg  = qcc.options("bg.margin"), 
-                cex = qcc.options("cex"),
-                mar = if(add.stats) 
-                            c(9+is.null(center)*-1, 2, 4, 2) + 0.1
-                      else  par("mar"),
-                no.readonly = TRUE)
-  if (restore.par) on.exit(par(oldpar))
-
+  oldpar <- par(no.readonly = TRUE)
+  if(restore.par) on.exit(par(oldpar))
+  par(bg  = qcc.options("bg.margin"), 
+      cex = oldpar$cex * qcc.options("cex"),
+      mar = if(add.stats) 
+              c(9+is.null(center)*-1, 2, 4, 2) + 0.1
+            else  oldpar$mar)
+  
   plot(0, 0, type="n", xlim = xlim, ylim = ylim,
        axes = FALSE, ylab="", xlab = "", main = title)
   usr <- par()$usr
@@ -102,9 +121,9 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
   abline(v=c(LSL,USL), col=2, lty=3, lwd=2)
   text(LSL, usr[4], "LSL", pos=1, col="darkgray", cex=0.8)
   text(USL, usr[4], "USL", pos=1, col="darkgray", cex=0.8)
-  if (!is.null(target))
-     { abline(v=target, col=2, lty=2, lwd=2)
-       text(target, usr[4], "Target", pos=1, col="darkgray", cex=0.8) }
+  if(has.target)
+    { abline(v=target, col=2, lty=2, lwd=2)
+      text(target, usr[4], "Target", pos=1, col="darkgray", cex=0.8) }
   lines(xx, dx, lty=2)
 
   if(add.stats) 
@@ -118,70 +137,67 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
       mtext(paste("Number of obs = ", n, sep = ""), 
             side = 1, line = 3, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Center = ", signif(center, options()$digits), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Center = ", signif(center, digits), sep = ""), 
             side = 1, line = 4, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("StdDev = ", signif(std.dev, options()$digits), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("StdDev = ", signif(std.dev, digits), sep = ""), 
             side = 1, line = 5, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       #--
-      if(!is.null(target))
-         msg <- paste("Target = ", signif(target, options()$digits), sep = "")
-      else
-         msg <- paste("Target = ", sep = "")     
-      mtext(msg, side = 1, line = 3, adj = 0, at = at.col[2],
+      mtext(ifelse(has.target, paste("Target = ", signif(target, digits), sep = ""),
+                               paste("Target = ")),
+            side = 1, line = 3, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("LSL = ", signif(LSL, options()$digits), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("LSL = ", ifelse(is.na(LSL), "", signif(LSL, digits)), sep = ""), 
             side = 1, line = 4, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("USL = ", signif(USL, options()$digits), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("USL = ", ifelse(is.na(USL), "", signif(USL, digits)), sep = ""), 
             side = 1, line = 5, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       #--
-      mtext(paste("Cp     = ", signif(Cp, 3), sep = ""), 
+      mtext(paste("Cp     = ", ifelse(is.na(Cp), "", signif(Cp, 3)), sep = ""), 
             side = 1, line = 3, adj = 0, at = at.col[3],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Cp_l  = ", signif(Cp.l, 3), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Cp_l  = ", ifelse(is.na(Cp.l), "", signif(Cp.l, 3)), sep = ""), 
             side = 1, line = 4, adj = 0, at = at.col[3],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Cp_u = ", signif(Cp.u, 3), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Cp_u = ", ifelse(is.na(Cp.u), "", signif(Cp.u, 3)), sep = ""), 
             side = 1, line = 5, adj = 0, at = at.col[3],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Cp_k = ", signif(Cp.k, 3), sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Cp_k = ", ifelse(is.na(Cp.k), "", signif(Cp.k, 3)), sep = ""), 
             side = 1, line = 6, adj = 0, at = at.col[3],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      if(!is.null(target))
-         mtext(paste("Cpm  = ", signif(Cpm, 3), sep = ""), 
-               side = 1, line = 7, adj = 0, at = at.col[3],
-               font = qcc.options("font.stats"),
-               cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Cpm  = ", ifelse(is.na(Cpm), "", signif(Cpm, 3)), sep = ""), 
+            side = 1, line = 7, adj = 0, at = at.col[3],
+            font = qcc.options("font.stats"),
+            cex = par("cex")*qcc.options("cex.stats"))
       #--
-      mtext(paste("Exp<LSL ", signif(exp.LSL, 2), "%", sep = ""), 
+      mtext(paste("Exp<LSL ", ifelse(is.na(exp.LSL), "", paste(signif(exp.LSL, 2), "%", sep="")), sep = ""), 
             side = 1, line = 3, adj = 0, at = at.col[4],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Exp>USL ", signif(exp.USL, 2), "%", sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Exp>USL ", ifelse(is.na(exp.USL), "", paste(signif(exp.USL, 2), "%", sep="")), sep = ""),
             side = 1, line = 4, adj = 0, at = at.col[4],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Obs<LSL ", signif(obs.LSL, 2), "%", sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Obs<LSL ", ifelse(is.na(obs.LSL), "", paste(signif(obs.LSL, 2), "%", sep="")), sep = ""), 
             side = 1, line = 5, adj = 0, at = at.col[4],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
-      mtext(paste("Obs>USL ", signif(obs.USL, 2), "%", sep = ""), 
+            cex = par("cex")*qcc.options("cex.stats"))
+      mtext(paste("Obs>USL ", ifelse(is.na(obs.USL), "", paste(signif(obs.USL, 2), "%", sep="")), sep = ""), 
             side = 1, line = 6, adj = 0, at = at.col[4],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
     }
 
   if(print)
@@ -190,38 +206,42 @@ process.capability <- function(object, spec.limits, target, std.dev, nsigmas, co
       cat(paste(formatC("Number of obs = ", width=16), 
                 formatC(n, width=12, flag="-"), 
                 formatC("Target = ", width=10), 
-                formatC(target, digits=options()$digits, flag="-"),
+                ifelse(has.target, formatC(signif(target,digits=digits), flag="-"), ""),
                 "\n", sep=""))
       cat(paste(formatC("Center = ", width=16), 
-                formatC(center, digits=options()$digits, width=12, flag="-"),
+                formatC(signif(center, digits=digits), width=12, flag="-"),
                 formatC("LSL = ", width=10), 
-                formatC(LSL, digits=options()$digits, flag="-"),
+                ifelse(is.na(LSL), "", formatC(signif(LSL, digits=digits), flag="-")),
                 "\n", sep=""))
       cat(paste(formatC("StdDev = ", width=16), 
-                formatC(std.dev, digits=options()$digits, width=12, flag="-"),
+                formatC(signif(std.dev, digits=digits), width=12, flag="-"),
                 formatC("USL = ", width=10), 
-                formatC(USL, digits=options()$digits, flag="-"),
+                ifelse(is.na(USL), "", formatC(signif(USL, digits=digits), flag="-")),
                 "\n", sep=""))
       cat("\nCapability indices:\n\n")
       print(tab, digits=4, na.print="", print.gap=2)
       cat("\n")
-      cat(paste("Exp<LSL ", format(exp.LSL, digits=2), "%   ", 
-                "Obs<LSL ", format(obs.LSL, digits=2), "% \n", sep=""))
-      cat(paste("Exp>USL ", format(exp.USL, digits=2), "%   ", 
-                "Obs>USL ", format(obs.USL, digits=2), "% \n", sep=""))
+      cat(paste("Exp<LSL", ifelse(is.na(exp.LSL), "\t", 
+                                   paste(format(exp.LSL, digits=2), "%\t", sep="")), 
+                "Obs<LSL", ifelse(is.na(obs.LSL), "", 
+                                   paste(format(obs.LSL, digits=2), "%\n", sep=""))))
+      cat(paste("Exp>USL", ifelse(is.na(exp.USL), "\t", 
+                                   paste(format(exp.USL, digits=2), "%\t", sep="")),
+                "Obs>USL", ifelse(is.na(obs.USL), "", 
+                                   paste(format(obs.USL, digits=2), "%\n", sep=""))))
     }
 
   invisible(list(nobs = n, center = center, std.dev = std.dev, 
-                 target=target, 
+                 target = target, 
                  spec.limits = { sl <- c(LSL, USL)
                                  names(sl) <- c("LSL", "USL")
                                  sl },
                  indices = tab, 
                  exp = { exp <- c(exp.LSL, exp.USL)/100
-                         names(exp) <- c("< LSL", "> USL")
+                         names(exp) <- c("Exp < LSL", "Exp > USL")
                          exp }, 
                  obs = { obs <- c(obs.LSL, obs.USL)/100
-                         names(obs) <- c("< LSL", "> USL")
+                         names(obs) <- c("Obs < LSL", "Obs > USL")
                          obs }
                  ))
 }
